//**************************************************************************//
//**                                                                      **//
//**                            TM6502 Component                          **//
//**                                                                      **//
//**   This component emulates a 6502 CPU.                                 **//
//**                                                                      **//
//**   Copyright (C) Dott. Cosimo De Michele 1998                         **//
//**         You are not allowed to distribuite this software             **//
//**         commercially. Please, notify me, if you make any             **//
//**         changes to this file.                                        **//
//**                                                                      **//
//**************************************************************************//

unit M6502;

interface

uses Classes, SysUtils;

const
  CPUcPVB = 180; //26500;

  // 6502 status flags :
  C_FLAG = $01; // Carry occurred
  Z_FLAG = $02; // Result is zero
  I_FLAG = $04; // Interrupts disabled
  D_FLAG = $08; // Decimal mode
  B_FLAG = $10; // Break [0 on stack after int]
  R_FLAG = $20; // Always 1
  V_FLAG = $40; // Overflow occured
  N_FLAG = $80; // Result is negative

  INT_NMI_ADDR    = $FFFA;
  INT_RESET_ADDR  = $FFFC;
  INT_IRQ_ADDR    = $FFFE;

type
  TOnWriteMemory = procedure (Addr : word; value : byte) of object;
  TOnReadMemoryByte  = function (Addr : word) : byte of object;
  TOnReadMemoryWord  = function (Addr : word) : Word of object;

  TM6502 = Class(TComponent)
  private
    sum,
    saveflags : integer;
    value     : byte;
    savepc   : word;

    addrMode : array [0..$FF] of procedure of object;
    Ticks    : array [0..$FF] of integer;
    Used     : array [0..$FF] of integer;
    Instruction : array [0..$FF] of procedure of object;

    function GetTPS: integer;
  protected
    CPUticks,
    LastTimerTick,
    timerTicks,
    clockticks : integer;
    { Addressing Modes }
    procedure Implied;
    procedure Immediate;
    procedure Abs;
    procedure Relative;
    procedure Indirect;
    procedure AbsX;
    procedure AbsY;
    procedure Zp;
    procedure ZpX;
    procedure ZpY;
    procedure IndX;
    procedure IndY;
    procedure IndAbsX;
    procedure IndZp;
    { Instruction }
    procedure adc;
    procedure _and;
    procedure asl;
    procedure asla;
    procedure bcc;
    procedure bcs;
    procedure beq;
    procedure bit;
    procedure bmi;
    procedure bne;
    procedure bpl;

    procedure bvc;
    procedure bvs;
    procedure clc;
    procedure cld;
    procedure cli;
    procedure clv;
    procedure cmp;
    procedure cpx;
    procedure cpy;
    { decrement instruction }
    procedure dec;
    procedure dex;
    procedure dey;
    procedure eor;
    procedure inc;
    procedure inx;
    procedure iny;
    procedure jmp;
    procedure jsr;
    { load instructions }
    procedure lda;
    procedure ldx;
    procedure ldy;

    procedure lsr;
    procedure lsra;
    procedure nop;
    procedure ora;
    procedure pha;
    procedure php;
    procedure pla;
    procedure plp;
    procedure rol;
    procedure rola;
    procedure ror;
    procedure rora;
    procedure rti;
    procedure rts;
    procedure sbc;
    procedure sec;
    procedure sed;
    procedure sei;
    { store instruction }
    procedure sta;
    procedure stx;
    procedure sty;
    procedure brk;
    procedure tax;
    procedure tay;
    procedure tsx;
    procedure txa;
    procedure txs;
    procedure tya;
    procedure bra;
    procedure dea;
    procedure ina;
    procedure phx;
    procedure plx;
    procedure phy;
    procedure ply;
    procedure stz;
    procedure tsb;
    procedure trb;
    function MemoryReadWord(Addr : word) : word;
  public
    S2 : integer;
    A,
    X,
    Y,
    P,
    S : byte;
    PC : word;
    OpCode : byte;
    ForcePause,
    CPUisRunning : boolean;
    MemoryWrite       : TOnWriteMemory;
    MemoryReadByte    : TOnReadMemoryByte;
    MemoryReadOpcode  : TOnReadMemoryByte;
    OnLoop,
    OnStop  : procedure of object;
  published
    property TPS           : integer read GetTPS;
    constructor Create(Owner: TComponent); override;
    function DAsm(var addr : word) : string;
    procedure Execute;
    procedure Reset;
    procedure Run;
    procedure Pause;
    procedure Stop;
    procedure NMI;
    procedure IRQ;
    procedure Save;
  end;

  procedure Register;

implementation

{$R M6502.DCR}

uses windows;

//**************************************************************************//
//**                                                                      **//
//**                       TM6502 component                               **//
//**                        the CPU engine.                               **//
//**                                                                      **//
//**************************************************************************//

function TM6502.GetTPS: integer;
var
  t : integer;
begin
  t := GetTickCount;
  if (CPUisRunning) and (LastTimerTick<>0) and (t<>LastTimerTick) then
    Result := (CPUticks * 1000) div (t - LastTimerTick)
  else
    Result := 0;
  LastTimerTick := t;
  CPUticks := 0;
end;

////////////////////////////////////////////////////////////////////////
//                                                                    //
//   The 6502 has 13 addressing modes, or ways of accessing memory.   //
//                                                                    //
//                   ADDRESSING MODE                                  //
//                                                                    //
////////////////////////////////////////////////////////////////////////

{IMPLIED}
procedure TM6502.Implied; assembler;
asm
end;

{IMMEDIATE}
procedure TM6502.Immediate;
begin
 savepc := PC;
 system.inc(PC);
end;

{ABSOLUTE}
procedure TM6502.abs;
begin
 savepc := MemoryReadWord(PC);
 system.inc(PC, 2);
end;

{BRANCH}
procedure TM6502.relative;
begin
 savepc := MemoryReadOpcode(PC);
 system.inc(PC);
 if (savepc and N_FLAG) <> 0 then
   system.dec(savepc, $100);
 if Hi(savepc) <> Hi(PC) then
   system.inc(clockticks);
end;

{INDIRECT ABSOLUTE}
procedure TM6502.indirect;
begin
 savepc := MemoryReadWord(MemoryReadWord(PC));
 system.inc(PC, 2);
end;

{ABSOLUTE INDEXED, X}
procedure TM6502.absx;
begin
 savepc := MemoryReadWord(PC);
 system.inc(PC,2);
 if (Ticks[opcode]=4) and (X<>0) then
   system.inc(clockticks);
 system.inc(savepc, X);
end;

{ABSOLUTE INDEXED, Y}
procedure TM6502.absy;
begin
 savepc := MemoryReadWord(PC);
 system.inc(PC,2);
 if (Ticks[opcode]=4) and (Y <> 0) then
   system.inc(clockticks);
 system.inc(savepc, Y);
end;

{ZERO PAGE}
procedure TM6502.zp;
begin
 savepc := MemoryReadOpcode(PC);
 system.inc(PC);
end;

{ZERO PAGE INDEXED, X}
procedure TM6502.zpx;
begin
 savepc:=MemoryReadOpcode(PC) + X;
 system.inc(PC);
 savepc := savepc and $FF;
end;

{ZERO PAGE INDEXED, Y}
procedure TM6502.zpy;
begin
 savepc:=MemoryReadOpcode(PC) + Y;
 system.inc(PC);
 savepc := savepc and $FF;
end;

{INDEXED INDIRECT, X}
procedure TM6502.indx;
begin
 value := MemoryReadOpcode(PC) + X;
 system.inc(PC);
 savepc := MemoryReadWord(value);
end;

{INDIRECT INDEXED, Y}
procedure TM6502.indy;
begin
 value := MemoryReadOpcode(PC);
 system.inc(PC);
 savepc := MemoryReadWord(value);
 if Ticks[opcode]=5 then
  if Y <> 0 then
   system.inc(clockticks);
 system.inc(savepc, Y);
end;

{(ABS,X)}
procedure TM6502.indabsx;
begin
 savepc := MemoryReadWord((MemoryReadWord(PC) + X));
end;

{(ZP)}
procedure TM6502.indzp;
begin
 value := MemoryReadOpcode(PC);
 system.inc(PC);
 savepc := MemoryReadWord(value);
end;

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                              Instructions                          //
//                                                                    //
////////////////////////////////////////////////////////////////////////

procedure TM6502.adc;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 saveflags := (P and $01);
 sum := A + value + saveflags;
 if (sum > $7F) or (sum < -$80) then
   P := P or $40
 else
   P := P and $BF;
 if sum > $FF then
   P := P or $1
 else
   P := P and $FE;
 A := sum;
 if (P and D_FLAG) <> 0 then
 begin
  P := P and $FE;
  if A and $0F > $9 then
    system.inc(A, $6);
  if A and $F0 > $90 then
  begin
   system.inc(A, $60);
   P := P or $01;
  end;
 end
 else
  system.inc(clockticks);
 if A <> 0 then
   P := P and $FD
 else
   P := P or $2;
 if A and $80 <> 0 then
   P := P or $80
 else
   P := P and $7F;
end;

procedure TM6502._and;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 A := A and value;
 if A <> 0 then
   P := P and $FD
 else
   P := P or $2;
 if A and $80 <> 0 then
   P := P or $80
 else
   P := P and $7F;
end;

procedure TM6502.asl;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 P := (P and $FE) or ((value shr 7) and $1);
 value := value shl 1;
 MemoryWrite(savepc, value);
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.asla;
begin
 P := (P and $FE) or ((A shr 7) and $1);
 A := A shl 1;
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.bcc;
begin
 if (P and $1) = 0 then begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.bcs;
begin
 if (P and $1) <> 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.beq;
begin
 if (P and Z_FLAG) <> 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.bit;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 if value and A <> 0 then P := P and $FD else P := P or $2;
 P := (P and $3F) or (value and $C0);
end;

procedure TM6502.bmi;
begin
 if (P and $80) <> 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.bne;
begin
 if (P and $2) = 0 then begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.bpl;
begin
 if (P and N_FLAG) = 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.brk;
begin
 system.inc(PC);
 MemoryWrite(S2, Hi(PC)); //BYTE(PC shr 8));
 system.dec(S2);
 MemoryWrite(S2, Lo(PC)); //BYTE(PC and $FF));
 system.dec(S2);
 MemoryWrite(S2, P);
 system.dec(S2);
 P := P or $14;
 PC := MemoryReadWord(INT_IRQ_ADDR);
end;


procedure TM6502.bvc;
begin
 if (P and $40) = 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.bvs;
begin
 if (P and $40) <> 0 then
 begin
  addrmode[opcode];
  system.inc(PC, savepc);
  system.inc(clockticks);
 end
 else
 begin
  value := MemoryReadByte(PC);
  system.inc(PC);
 end;
end;

procedure TM6502.clc;
begin
  P := P and not(C_FLAG);
end;


procedure TM6502.cld;
begin
  P := P and not(D_FLAG);
end;

procedure TM6502.cli;
begin
  P := P and not(I_FLAG);
end;

procedure TM6502.clv;
begin
  P := P and not(V_FLAG);
end;

procedure TM6502.cmp;
begin
 addrmode[opcode];
 value := A + $100 - MemoryReadByte(savepc);
 if A + $100 - MemoryReadByte(savepc) > $FF then P := P or $1 else P := P and $FE;
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.cpx;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 if X + $100 - value > $FF then P := P or $1 else P := P and $FE;
 value := X + $100 - value;
 if value <> 0 then P := P and $FD else P := P or $02;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.cpy;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 if Y + $100 - value > $FF then P := P or $1 else P := P and $FE;
 value := Y + $100 - value;
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.dec;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 system.dec(value);
 MemoryWrite(savepc, value);
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.dex;
begin
 system.dec(X);
 if (X <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (X and N_FLAG) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.dey;
begin
 system.dec(Y);
 if (Y <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (Y and $80) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.eor;
begin
 addrmode[opcode];
 A := A xor MemoryReadByte(savepc);
 if A <> 0 then P := P and $FD else P := P or $2;
 if (A and $80) <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.inc;
begin
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 system.inc(value);
 MemoryWrite(savepc, value);
 if (value <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (value and $80) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.inx;
begin
 system.inc(X);
 if (X <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (X and N_FLAG) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.iny;
begin
 system.inc(Y);
 if (Y <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (Y and $80) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.jmp;
begin
 addrmode[opcode];
 PC:=savepc;
end;

procedure TM6502.jsr;
begin
 system.inc(PC);
 MemoryWrite(S2, Hi(PC));
 system.dec(S2);
 MemoryWrite(S2, Lo(PC));
 system.dec(S2);
 system.dec(PC);
 addrmode[opcode];
 PC:=savepc;
end;

procedure TM6502.lda;
begin
 addrmode[opcode];
 A:=MemoryReadByte(savepc);
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.ldx;
begin
 addrmode[opcode];
 X:=MemoryReadByte(savepc);
 if X <> 0 then P := P and $FD else P := P or $2;
 if X and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.ldy;
begin
 addrmode[opcode];
 Y:=MemoryReadByte(savepc);
 if Y <> 0 then P := P and $FD else P := P or $2;
 if Y and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.lsr;
begin
 addrmode[opcode];
 value:=MemoryReadByte(savepc);
 P := (P and $FE) or (value and $1);
 value := value shr 1;
 MemoryWrite(savepc, value);
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 = $80 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.lsra;
begin
 P := (P and $FE) or (A and $1);
 A := A shr 1;
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.nop; assembler;
asm
end;

procedure TM6502.ora;
begin
 addrmode[opcode];
 A := A or MemoryReadByte(savepc);
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.pha;
begin
 MemoryWrite(S2, A);
 {MemoryWrite($100+S, A);}
 system.dec(S2);
end;

procedure TM6502.php;
begin
 MemoryWrite(S2, P);
 {MemoryWrite($100+S, A);}
 system.dec(S2);
end;

procedure TM6502.pla;
begin
 system.inc(S2);
 A := MemoryReadByte(S2);
 if A <> 0 then P := P and $FD else P := P or $02;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.plp;
begin
  system.inc(S2);
  P := MemoryReadByte(S2) or R_FLAG;
end;

procedure TM6502.rol;
begin
 saveflags:=(P and $1);
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 P := (P and $FE) or ((value shr 7) and $1);
 value := value shl 1;
 value := value or saveflags;
 MemoryWrite(savepc, value);
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.rola;
begin
 saveflags:=(P and $1);
 P := (P and $FE) or ((A shr 7) and $1);
 A := A shl 1;
 A := A or saveflags;
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.ror;
begin
 saveflags:=(P and $1);
 addrmode[opcode];
 value := MemoryReadByte(savepc);
 P := (P and $FE) or (value and $1);
 value := value shr 1;
 if saveflags <> 0 then value := value or $80;
 MemoryWrite(savepc, value);
 if value <> 0 then P := P and $FD else P := P or $2;
 if value and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.rora;
begin
 saveflags:=(P and $1);
 P := (P and $FE) or (A and $1);
 A := A shr 1;
 if saveflags <> 0 then A := A or $80;
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.rti;
begin
 system.inc(S2);
 P := MemoryReadByte(S2) or $20;
 system.inc(S2);
 PC := MemoryReadByte(S2);
 system.inc(S2);
 PC := PC or (MemoryReadByte(S2) shl 8);
end;

procedure TM6502.rts;
begin
 system.inc(S2);
 PC := MemoryReadByte(S2);
 system.inc(S2);
 PC := PC or (MemoryReadByte(S2) shl 8);
 system.inc(PC);
end;

procedure TM6502.sbc;
begin
 addrmode[opcode];
 value :=  MemoryReadByte(savepc) xor $FF;
 saveflags := P and $1;
 sum := A + value + (saveflags shl 4);
 if (sum>$7F) or (sum<-$80) then P := P or $40 else P := P and $BF;
 sum := A + value + saveflags;
 if sum > $FF then P := P or $01 else P := P and $FE;
 A := sum;
 if P and $8 <> 0 then begin
  system.dec(A,$66);
  P := P and $FE;
  if A and $0F > $9 then system.inc(A, $6);
  if A and $F0 > $90 then begin
   system.inc(A,$60);
   P := P or $1;
  end;
 end
 else
  system.inc(clockticks);
 if A <> 0 then P := P and $FD else P := P or $2;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.sec;
begin
  P := P or C_FLAG;
end;

procedure TM6502.sed;
begin
  P := P or D_FLAG;
end;

procedure TM6502.sei;
begin
  P := P or I_FLAG;
end;

procedure TM6502.sta;
begin
 addrmode[opcode];
 MemoryWrite(savepc, A);
end;

procedure TM6502.stx;
begin
 addrmode[opcode];
 MemoryWrite(savepc, X);
end;

procedure TM6502.sty;
begin
 addrmode[opcode];
 MemoryWrite(savepc, Y);
end;

procedure TM6502.tax;
begin
 X := A;
 if (X <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (X and $80) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.tay;
begin
 Y := A;
 if (Y <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (Y and $80) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.tsx;
begin
 X := S;
 if (X <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (X and N_FLAG) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.txa;
begin
 A := X;
 if (A <> 0) then
   P := P and not(Z_FLAG)
 else
   P := P or Z_FLAG;
 if (A and N_FLAG) <> 0 then
   P := P or N_FLAG
 else
   P := P and not(N_FLAG);
end;

procedure TM6502.txs;
begin
  S2 := X + $100;
end;

procedure TM6502.tya;
begin
 A := Y;
 if A <> 0 then P := P and $FD else P := P or $02;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.bra;
begin
 addrmode[opcode];
 system.inc(PC, savepc);
 system.inc(clockticks);
end;

procedure TM6502.dea;
begin
 system.dec(A);
 if A <> 0 then P := P and $FD else P := P or $02;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.ina;
begin
 system.inc(A);
 if A <> 0 then P := P and $FD else P := P or $02;
 if A and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.phx;
begin
 {MemoryWrite($100+S, X);}
 MemoryWrite(S2, X);
 system.dec(S2);
end;

procedure TM6502.plx;
begin
 system.inc(S2);
 X := MemoryReadByte(S2);
 if X <> 0 then P := P and $FD else P := P or $2;
 if X and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.phy;
begin
 {MemoryWrite($100+S, Y);}
 MemoryWrite(S2, Y);
 system.dec(S2);
end;

procedure TM6502.ply;
begin
 system.inc(S2);
 Y := MemoryReadByte(S2);
 if Y <> 0 then P := P and $FD else P := P or $2;
 if Y and $80 <> 0 then P := P or $80 else P := P and $7F;
end;

procedure TM6502.stz;
begin
 addrmode[opcode];
 {MemoryWrite(savepc, 0);}
 MemoryWrite(savepc, 0);
end;

procedure TM6502.tsb;
begin
 addrmode[opcode];
 MemoryWrite(savepc, A);
 {MemoryWrite(savepc, Memory[savepc] or A);}
 if MemoryReadByte(savepc) <> 0 then
   P := P and $FD
 else
   P := P or $02;
end;

procedure TM6502.trb;
begin
 addrmode[opcode];
 MemoryWrite(savepc, MemoryReadByte(savepc) and (A xor $FF));
 {MemoryWrite(savepc, Memory[savepc] and (A xor $FF));}
 if MemoryReadByte(savepc) <> 0 then
   P := P and $FD
 else
   P := P or $02;
end;

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                   Methods to operate                               //
//                                                                    //
////////////////////////////////////////////////////////////////////////

constructor TM6502.Create(Owner: TComponent);
begin
  inherited;
  { inizializzo istruzioni }
  ticks[$00]:=7; instruction[$00] := brk; addrMode[$00]:=implied;
  ticks[$01]:=6; instruction[$01] := ora; addrMode[$01]:=indx;
  ticks[$02]:=2; instruction[$02] := nop; addrMode[$02]:=implied;
  ticks[$03]:=2; instruction[$03] := nop; addrMode[$03]:=implied;
  ticks[$04]:=3; instruction[$04] := tsb; addrMode[$04]:=zp;
  ticks[$05]:=3; instruction[$05] := ora; addrMode[$05]:=zp;
  ticks[$06]:=5; instruction[$06] := asl; addrMode[$06]:=zp;
  ticks[$07]:=2; instruction[$07] := nop; addrMode[$07]:=implied;
  ticks[$08]:=3; instruction[$08] := php; addrMode[$08]:=implied;
  ticks[$09]:=3; instruction[$09] := ora; addrMode[$09]:=immediate;
  ticks[$0A]:=2; instruction[$0A] := asla; addrMode[$0A]:=implied;
  ticks[$0B]:=2; instruction[$0B] := nop; addrMode[$0B]:=implied;
  ticks[$0C]:=4; instruction[$0C] := tsb; addrMode[$0C]:=abs;
  ticks[$0D]:=4; instruction[$0D] := ora; addrMode[$0D]:=abs;
  ticks[$0E]:=6; instruction[$0E] := asl; addrMode[$0E]:=abs;
  ticks[$0F]:=2; instruction[$0F] := nop; addrMode[$0F]:=implied;
  ticks[$10]:=2; instruction[$10] := bpl; addrMode[$10]:=relative;
  ticks[$11]:=5; instruction[$11] := ora; addrMode[$11]:=indy;
  ticks[$12]:=3; instruction[$12] := ora; addrMode[$12]:=indzp;
  ticks[$13]:=2; instruction[$13] := nop; addrMode[$13]:=implied;
  ticks[$14]:=3; instruction[$14] := trb; addrMode[$14]:=zp;
  ticks[$15]:=4; instruction[$15] := ora; addrMode[$15]:=zpx;
  ticks[$16]:=6; instruction[$16] := asl; addrMode[$16]:=zpx;
  ticks[$17]:=2; instruction[$17] := nop; addrMode[$17]:=implied;
  ticks[$18]:=2; instruction[$18] := clc; addrMode[$18]:=implied;
  ticks[$19]:=4; instruction[$19] := ora; addrMode[$19]:=absy;
  ticks[$1A]:=2; instruction[$1A] := ina; addrMode[$1A]:=implied;
  ticks[$1B]:=2; instruction[$1B] := nop; addrMode[$1B]:=implied;
  ticks[$1C]:=4; instruction[$1C] := trb; addrMode[$1C]:=abs;
  ticks[$1D]:=4; instruction[$1D] := ora; addrMode[$1D]:=absx;
  ticks[$1E]:=7; instruction[$1E] := asl; addrMode[$1E]:=absx;
  ticks[$1F]:=2; instruction[$1F] := nop; addrMode[$1F]:=implied;
  ticks[$20]:=6; instruction[$20] := jsr; addrMode[$20]:=abs;
  ticks[$21]:=6; instruction[$21] := _and; addrMode[$21]:=indx;
  ticks[$22]:=2; instruction[$22] := nop; addrMode[$22]:=implied;
  ticks[$23]:=2; instruction[$23] := nop; addrMode[$23]:=implied;
  ticks[$24]:=3; instruction[$24] := bit; addrMode[$24]:=zp;
  ticks[$25]:=3; instruction[$25] := _and; addrMode[$25]:=zp;
  ticks[$26]:=5; instruction[$26] := rol; addrMode[$26]:=zp;
  ticks[$27]:=2; instruction[$27] := nop; addrMode[$27]:=implied;
  ticks[$28]:=4; instruction[$28] := plp; addrMode[$28]:=implied;
  ticks[$29]:=3; instruction[$29] := _and; addrMode[$29]:=immediate;
  ticks[$2A]:=2; instruction[$2A] := rola; addrMode[$2A]:=implied;
  ticks[$2B]:=2; instruction[$2B] := nop; addrMode[$2B]:=implied;
  ticks[$2C]:=4; instruction[$2C] := bit; addrMode[$2C]:=abs;
  ticks[$2D]:=4; instruction[$2D] := _and; addrMode[$2D]:=abs;
  ticks[$2E]:=6; instruction[$2E] := rol; addrMode[$2E]:=abs;
  ticks[$2F]:=2; instruction[$2F] := nop; addrMode[$2F]:=implied;
  ticks[$30]:=2; instruction[$30] := bmi; addrMode[$30]:=relative;
  ticks[$31]:=5; instruction[$31] := _and; addrMode[$31]:=indy;
  ticks[$32]:=3; instruction[$32] := _and; addrMode[$32]:=indzp;
  ticks[$33]:=2; instruction[$33] := nop; addrMode[$33]:=implied;
  ticks[$34]:=4; instruction[$34] := bit; addrMode[$34]:=zpx;
  ticks[$35]:=4; instruction[$35] := _and; addrMode[$35]:=zpx;
  ticks[$36]:=6; instruction[$36] := rol; addrMode[$36]:=zpx;
  ticks[$37]:=2; instruction[$37] := nop; addrMode[$37]:=implied;
  ticks[$38]:=2; instruction[$38] := sec; addrMode[$38]:=implied;
  ticks[$39]:=4; instruction[$39] := _and; addrMode[$39]:=absy;
  ticks[$3A]:=2; instruction[$3A] := dea; addrMode[$3A]:=implied;
  ticks[$3B]:=2; instruction[$3B] := nop; addrMode[$3B]:=implied;
  ticks[$3C]:=4; instruction[$3C] := bit; addrMode[$3C]:=absx;
  ticks[$3D]:=4; instruction[$3D] := _and; addrMode[$3D]:=absx;
  ticks[$3E]:=7; instruction[$3E] := rol; addrMode[$3E]:=absx;
  ticks[$3F]:=2; instruction[$3F] := nop; addrMode[$3F]:=implied;
  ticks[$40]:=6; instruction[$40] := rti; addrMode[$40]:=implied;
  ticks[$41]:=6; instruction[$41] := eor; addrMode[$41]:=indx;
  ticks[$42]:=2; instruction[$42] := nop; addrMode[$42]:=implied;
  ticks[$43]:=2; instruction[$43] := nop; addrMode[$43]:=implied;
  ticks[$44]:=2; instruction[$44] := nop; addrMode[$44]:=implied;
  ticks[$45]:=3; instruction[$45] := eor; addrMode[$45]:=zp;

  ticks[$46]:=5; instruction[$46] := lsr; addrMode[$46]:=zp;
  ticks[$47]:=2; instruction[$47] := nop; addrMode[$47]:=implied;
  ticks[$48]:=3; instruction[$48] := pha; addrMode[$48]:=implied;
  ticks[$49]:=3; instruction[$49] := eor; addrMode[$49]:=immediate;
  ticks[$4A]:=2; instruction[$4A] := lsra; addrMode[$4A]:=implied;
  ticks[$4B]:=2; instruction[$4B] := nop; addrMode[$4B]:=implied;
  ticks[$4C]:=3; instruction[$4C] := jmp; addrMode[$4C]:=abs;
  ticks[$4D]:=4; instruction[$4D] := eor; addrMode[$4D]:=abs;
  ticks[$4E]:=6; instruction[$4E] := lsr; addrMode[$4E]:=abs;
  ticks[$4F]:=2; instruction[$4F] := nop; addrMode[$4F]:=implied;
  ticks[$50]:=2; instruction[$50] := bvc; addrMode[$50]:=relative;
  ticks[$51]:=5; instruction[$51] := eor; addrMode[$51]:=indy;
  ticks[$52]:=3; instruction[$52] := eor; addrMode[$52]:=indzp;
  ticks[$53]:=2; instruction[$53] := nop; addrMode[$53]:=implied;
  ticks[$54]:=2; instruction[$54] := nop; addrMode[$54]:=implied;
  ticks[$55]:=4; instruction[$55] := eor; addrMode[$55]:=zpx;
  ticks[$56]:=6; instruction[$56] := lsr; addrMode[$56]:=zpx;
  ticks[$57]:=2; instruction[$57] := nop; addrMode[$57]:=implied;
  ticks[$58]:=2; instruction[$58] := cli; addrMode[$58]:=implied;
  ticks[$59]:=4; instruction[$59] := eor; addrMode[$59]:=absy;
  ticks[$5A]:=3; instruction[$5A] := phy; addrMode[$5A]:=implied;
  ticks[$5B]:=2; instruction[$5B] := nop; addrMode[$5B]:=implied;
  ticks[$5C]:=2; instruction[$5C] := nop; addrMode[$5C]:=implied;
  ticks[$5D]:=4; instruction[$5D] := eor; addrMode[$5D]:=absx;
  ticks[$5E]:=7; instruction[$5E] := lsr; addrMode[$5E]:=absx;
  ticks[$5F]:=2; instruction[$5F] := nop; addrMode[$5F]:=implied;
  ticks[$60]:=6; instruction[$60] := rts; addrMode[$60]:=implied;
  ticks[$61]:=6; instruction[$61] := adc; addrMode[$61]:=indx;
  ticks[$62]:=2; instruction[$62] := nop; addrMode[$62]:=implied;
  ticks[$63]:=2; instruction[$63] := nop; addrMode[$63]:=implied;
  ticks[$64]:=3; instruction[$64] := stz; addrMode[$64]:=zp;
  ticks[$65]:=3; instruction[$65] := adc; addrMode[$65]:=zp;
  ticks[$66]:=5; instruction[$66] := ror; addrMode[$66]:=zp;
  ticks[$67]:=2; instruction[$67] := nop; addrMode[$67]:=implied;
  ticks[$68]:=4; instruction[$68] := pla; addrMode[$68]:=implied;
  ticks[$69]:=3; instruction[$69] := adc; addrMode[$69]:=immediate;
  ticks[$6A]:=2; instruction[$6A] := rora; addrMode[$6A]:=implied;
  ticks[$6B]:=2; instruction[$6B] := nop; addrMode[$6B]:=implied;
  ticks[$6C]:=5; instruction[$6C] := jmp; addrMode[$6C]:=indirect;
  ticks[$6D]:=4; instruction[$6D] := adc; addrMode[$6D]:=abs;
  ticks[$6E]:=6; instruction[$6E] := ror; addrMode[$6E]:=abs;
  ticks[$6F]:=2; instruction[$6F] := nop; addrMode[$6F]:=implied;
  ticks[$70]:=2; instruction[$70] := bvs; addrMode[$70]:=relative;
  ticks[$71]:=5; instruction[$71] := adc; addrMode[$71]:=indy;
  ticks[$72]:=3; instruction[$72] := adc; addrMode[$72]:=indzp;
  ticks[$73]:=2; instruction[$73] := nop; addrMode[$73]:=implied;
  ticks[$74]:=4; instruction[$74] := stz; addrMode[$74]:=zpx;
  ticks[$75]:=4; instruction[$75] := adc; addrMode[$75]:=zpx;
  ticks[$76]:=6; instruction[$76] := ror; addrMode[$76]:=zpx;
  ticks[$77]:=2; instruction[$77] := nop; addrMode[$77]:=implied;
  ticks[$78]:=2; instruction[$78] := sei; addrMode[$78]:=implied;
  ticks[$79]:=4; instruction[$79] := adc; addrMode[$79]:=absy;
  ticks[$7A]:=4; instruction[$7A] := ply; addrMode[$7A]:=implied;
  ticks[$7B]:=2; instruction[$7B] := nop; addrMode[$7B]:=implied;
  ticks[$7C]:=6; instruction[$7C] := jmp; addrMode[$7C]:=indabsx;
  ticks[$7D]:=4; instruction[$7D] := adc; addrMode[$7D]:=absx;
  ticks[$7E]:=7; instruction[$7E] := ror; addrMode[$7E]:=absx;
  ticks[$7F]:=2; instruction[$7F] := nop; addrMode[$7F]:=implied;
  ticks[$80]:=2; instruction[$80] := bra; addrMode[$80]:=relative;
  ticks[$81]:=6; instruction[$81] := sta; addrMode[$81]:=indx;
  ticks[$82]:=2; instruction[$82] := nop; addrMode[$82]:=implied;
  ticks[$83]:=2; instruction[$83] := nop; addrMode[$83]:=implied;
  ticks[$84]:=2; instruction[$84] := sty; addrMode[$84]:=zp;
  ticks[$85]:=2; instruction[$85] := sta; addrMode[$85]:=zp;
  ticks[$86]:=2; instruction[$86] := stx; addrMode[$86]:=zp;
  ticks[$87]:=2; instruction[$87] := nop; addrMode[$87]:=implied;

  ticks[$88]:=2; instruction[$88] := dey; addrMode[$88]:=implied;
  ticks[$89]:=2; instruction[$89] := bit; addrMode[$89]:=immediate;
  ticks[$8A]:=2; instruction[$8A] := txa; addrMode[$8A]:=implied;
  ticks[$8B]:=2; instruction[$8B] := nop; addrMode[$8B]:=implied;
  ticks[$8C]:=4; instruction[$8C] := sty; addrMode[$8C]:=abs;
  ticks[$8D]:=4; instruction[$8D] := sta; addrMode[$8D]:=abs;
  ticks[$8E]:=4; instruction[$8E] := stx; addrMode[$8E]:=abs;
  ticks[$8F]:=2; instruction[$8F] := nop; addrMode[$8F]:=implied;
  ticks[$90]:=2; instruction[$90] := bcc; addrMode[$90]:=relative;
  ticks[$91]:=6; instruction[$91] := sta; addrMode[$91]:=indy;
  ticks[$92]:=3; instruction[$92] := sta; addrMode[$92]:=indzp;
  ticks[$93]:=2; instruction[$93] := nop; addrMode[$93]:=implied;
  ticks[$94]:=4; instruction[$94] := sty; addrMode[$94]:=zpx;
  ticks[$95]:=4; instruction[$95] := sta; addrMode[$95]:=zpx;
  ticks[$96]:=4; instruction[$96] := stx; addrMode[$96]:=zpy;
  ticks[$97]:=2; instruction[$97] := nop; addrMode[$97]:=implied;
  ticks[$98]:=2; instruction[$98] := tya; addrMode[$98]:=implied;
  ticks[$99]:=5; instruction[$99] := sta; addrMode[$99]:=absy;
  ticks[$9A]:=2; instruction[$9A] := txs; addrMode[$9A]:=implied;
  ticks[$9B]:=2; instruction[$9B] := nop; addrMode[$9B]:=implied;
  ticks[$9C]:=4; instruction[$9C] := stz; addrMode[$9C]:=abs;
  ticks[$9D]:=5; instruction[$9D] := sta; addrMode[$9D]:=absx;
  ticks[$9E]:=5; instruction[$9E] := stz; addrMode[$9E]:=absx;
  ticks[$9F]:=2; instruction[$9F] := nop; addrMode[$9F]:=implied;
  ticks[$A0]:=3; instruction[$A0] := ldy; addrMode[$A0]:=immediate;
  ticks[$A1]:=6; instruction[$A1] := lda; addrMode[$A1]:=indx;
  ticks[$A2]:=3; instruction[$A2] := ldx; addrMode[$A2]:=immediate;
  ticks[$A3]:=2; instruction[$A3] := nop; addrMode[$A3]:=implied;
  ticks[$A4]:=3; instruction[$A4] := ldy; addrMode[$A4]:=zp;
  ticks[$A5]:=3; instruction[$A5] := lda; addrMode[$A5]:=zp;
  ticks[$A6]:=3; instruction[$A6] := ldx; addrMode[$A6]:=zp;
  ticks[$A7]:=2; instruction[$A7] := nop; addrMode[$A7]:=implied;
  ticks[$A8]:=2; instruction[$A8] := tay; addrMode[$A8]:=implied;
  ticks[$A9]:=3; instruction[$A9] := lda; addrMode[$A9]:=immediate;
  ticks[$AA]:=2; instruction[$AA] := tax; addrMode[$AA]:=implied;
  ticks[$AB]:=2; instruction[$AB] := nop; addrMode[$AB]:=implied;
  ticks[$AC]:=4; instruction[$AC] := ldy; addrMode[$AC]:=abs;
  ticks[$AD]:=4; instruction[$AD] := lda; addrMode[$AD]:=abs;
  ticks[$AE]:=4; instruction[$AE] := ldx; addrMode[$AE]:=abs;
  ticks[$AF]:=2; instruction[$AF] := nop; addrMode[$AF]:=implied;
  ticks[$B0]:=2; instruction[$B0] := bcs; addrMode[$B0]:=relative;
  ticks[$B1]:=5; instruction[$B1] := lda; addrMode[$B1]:=indy;
  ticks[$B2]:=3; instruction[$B2] := lda; addrMode[$B2]:=indzp;
  ticks[$B3]:=2; instruction[$B3] := nop; addrMode[$B3]:=implied;
  ticks[$B4]:=4; instruction[$B4] := ldy; addrMode[$B4]:=zpx;
  ticks[$B5]:=4; instruction[$B5] := lda; addrMode[$B5]:=zpx;
  ticks[$B6]:=4; instruction[$B6] := ldx; addrMode[$B6]:=zpy;
  ticks[$B7]:=2; instruction[$B7] := nop; addrMode[$B7]:=implied;
  ticks[$B8]:=2; instruction[$B8] := clv; addrMode[$B8]:=implied;
  ticks[$B9]:=4; instruction[$B9] := lda; addrMode[$B9]:=absy;
  ticks[$BA]:=2; instruction[$BA] := tsx; addrMode[$BA]:=implied;
  ticks[$BB]:=2; instruction[$BB] := nop; addrMode[$BB]:=implied;
  ticks[$BC]:=4; instruction[$BC] := ldy; addrMode[$BC]:=absx;
  ticks[$BD]:=4; instruction[$BD] := lda; addrMode[$BD]:=absx;
  ticks[$BE]:=4; instruction[$BE] := ldx; addrMode[$BE]:=absy;
  ticks[$BF]:=2; instruction[$BF] := nop; addrMode[$BF]:=implied;
  ticks[$C0]:=3; instruction[$C0] := cpy; addrMode[$C0]:=immediate;
  ticks[$C1]:=6; instruction[$C1] := cmp; addrMode[$C1]:=indx;
  ticks[$C2]:=2; instruction[$C2] := nop; addrMode[$C2]:=implied;
  ticks[$C3]:=2; instruction[$C3] := nop; addrMode[$C3]:=implied;
  ticks[$C4]:=3; instruction[$C4] := cpy; addrMode[$C4]:=zp;
  ticks[$C5]:=3; instruction[$C5] := cmp; addrMode[$C5]:=zp;
  ticks[$C6]:=5; instruction[$C6] := dec; addrMode[$C6]:=zp;
  ticks[$C7]:=2; instruction[$C7] := nop; addrMode[$C7]:=implied;
  ticks[$C8]:=2; instruction[$C8] := iny; addrMode[$C8]:=implied;
  ticks[$C9]:=3; instruction[$C9] := cmp; addrMode[$C9]:=immediate;

  ticks[$CA]:=2; instruction[$CA] := dex; addrMode[$CA]:=implied;
  ticks[$CB]:=2; instruction[$CB] := nop; addrMode[$CB]:=implied;
  ticks[$CC]:=4; instruction[$CC] := cpy; addrMode[$CC]:=abs;
  ticks[$CD]:=4; instruction[$CD] := cmp; addrMode[$CD]:=abs;
  ticks[$CE]:=6; instruction[$CE] := dec; addrMode[$CE]:=abs;
  ticks[$CF]:=2; instruction[$CF] := nop; addrMode[$CF]:=implied;
  ticks[$D0]:=2; instruction[$D0] := bne; addrMode[$D0]:=relative;
  ticks[$D1]:=5; instruction[$D1] := cmp; addrMode[$D1]:=indy;
  ticks[$D2]:=3; instruction[$D2] := cmp; addrMode[$D2]:=indzp;
  ticks[$D3]:=2; instruction[$D3] := nop; addrMode[$D3]:=implied;
  ticks[$D4]:=2; instruction[$D4] := nop; addrMode[$D4]:=implied;
  ticks[$D5]:=4; instruction[$D5] := cmp; addrMode[$D5]:=zpx;
  ticks[$D6]:=6; instruction[$D6] := dec; addrMode[$D6]:=zpx;
  ticks[$D7]:=2; instruction[$D7] := nop; addrMode[$D7]:=implied;
  ticks[$D8]:=2; instruction[$D8] := cld; addrMode[$D8]:=implied;
  ticks[$D9]:=4; instruction[$D9] := cmp; addrMode[$D9]:=absy;
  ticks[$DA]:=3; instruction[$DA] := phx; addrMode[$DA]:=implied;
  ticks[$DB]:=2; instruction[$DB] := nop; addrMode[$DB]:=implied;
  ticks[$DC]:=2; instruction[$DC] := nop; addrMode[$DC]:=implied;
  ticks[$DD]:=4; instruction[$DD] := cmp; addrMode[$DD]:=absx;
  ticks[$DE]:=7; instruction[$DE] := dec; addrMode[$DE]:=absx;
  ticks[$DF]:=2; instruction[$DF] := nop; addrMode[$DF]:=implied;
  ticks[$E0]:=3; instruction[$E0] := cpx; addrMode[$E0]:=immediate;
  ticks[$E1]:=6; instruction[$E1] := sbc; addrMode[$E1]:=indx;
  ticks[$E2]:=2; instruction[$E2] := nop; addrMode[$E2]:=implied;
  ticks[$E3]:=2; instruction[$E3] := nop; addrMode[$E3]:=implied;
  ticks[$E4]:=3; instruction[$E4] := cpx; addrMode[$E4]:=zp;
  ticks[$E5]:=3; instruction[$E5] := sbc; addrMode[$E5]:=zp;
  ticks[$E6]:=5; instruction[$E6] := inc; addrMode[$E6]:=zp;
  ticks[$E7]:=2; instruction[$E7] := nop; addrMode[$E7]:=implied;
  ticks[$E8]:=2; instruction[$E8] := inx; addrMode[$E8]:=implied;
  ticks[$E9]:=3; instruction[$E9] := sbc; addrMode[$E9]:=immediate;
  ticks[$EA]:=2; instruction[$EA] := nop; addrMode[$EA]:=implied;
  ticks[$EB]:=2; instruction[$EB] := nop; addrMode[$EB]:=implied;
  ticks[$EC]:=4; instruction[$EC] := cpx; addrMode[$EC]:=abs;
  ticks[$ED]:=4; instruction[$ED] := sbc; addrMode[$ED]:=abs;
  ticks[$EE]:=6; instruction[$EE] := inc; addrMode[$EE]:=abs;
  ticks[$EF]:=2; instruction[$EF] := nop; addrMode[$EF]:=implied;
  ticks[$F0]:=2; instruction[$F0] := beq; addrMode[$F0]:=relative;
  ticks[$F1]:=5; instruction[$F1] := sbc; addrMode[$F1]:=indy;
  ticks[$F2]:=3; instruction[$F2] := sbc; addrMode[$F2]:=indzp;
  ticks[$F3]:=2; instruction[$F3] := nop; addrMode[$F3]:=implied;
  ticks[$F4]:=2; instruction[$F4] := nop; addrMode[$F4]:=implied;
  ticks[$F5]:=4; instruction[$F5] := sbc; addrMode[$F5]:=zpx;
  ticks[$F6]:=6; instruction[$F6] := inc; addrMode[$F6]:=zpx;
  ticks[$F7]:=2; instruction[$F7] := nop; addrMode[$F7]:=implied;
  ticks[$F8]:=2; instruction[$F8] := sed; addrMode[$F8]:=implied;
  ticks[$F9]:=4; instruction[$F9] := sbc; addrMode[$F9]:=absy;
  ticks[$FA]:=4; instruction[$FA] := plx; addrMode[$FA]:=implied;
  ticks[$FB]:=2; instruction[$FB] := nop; addrMode[$FB]:=implied;
  ticks[$FC]:=2; instruction[$FC] := nop; addrMode[$FC]:=implied;
  ticks[$FD]:=4; instruction[$FD] := sbc; addrMode[$FD]:=absx;
  ticks[$FE]:=7; instruction[$FE] := inc; addrMode[$FE]:=absx;
  ticks[$FF]:=2; instruction[$FF] := nop; addrMode[$FF]:=implied;

  CPUisRunning := false;
  ForcePause := false;
end; { Create }

procedure TM6502.Reset;
begin
  CPUisRunning := false;
  ForcePause := false;
  A:=0; X:=0; Y:=0; P:= Z_FLAG or R_FLAG;
  S := $FF;
  S2 := $FF + $100;
  PC := MemoryReadWord(INT_RESET_ADDR);
  timerTicks := CPUcPVB;
  clockticks := 0;
  CPUticks   := 0;
end; { Reset }

procedure TM6502.Execute;
begin
  ForcePause := false;
  CPUisRunning := true;
  system.inc(PC);
  { execute instruction }
  instruction[opcode];
  { calculate clock cycles }
  system.inc(clockticks, ticks[opcode]);
  system.dec(timerTicks, clockticks);
  system.inc(CPUticks, ticks[opcode]);
  clockticks := 0;
  { check for interrupts and do other hardware emulation }
  if timerTicks<0 then
  begin
    OnLoop;
    system.inc(timerTicks, CPUcPVB);
  end;
end; { Execute }

procedure TM6502.Run;
begin
  CPUisRunning := true;
  ForcePause := false;
  {CPUThread := TCPUThread.Create(Self);}
  repeat
    { fetch instruction }
    opcode := MemoryReadOpcode(PC);
    { execute opcode }
    system.inc(PC);
    { execute instruction }
    instruction[opcode];
    //system.inc(Used[opcode]);
    { calculate clock cycles }
    system.inc(clockticks, ticks[opcode]);
    system.dec(timerTicks, clockticks);
    system.inc(CPUticks, ticks[opcode]);
    clockticks := 0;
    { check for interrupts and do other hardware emulation }
    if timerTicks<0 then
    begin
      OnLoop;
      system.inc(timerTicks, CPUcPVB);
    end;
  until not CPUisRunning;
  if ForcePause then
    ForcePause := false
  else
    OnStop;
end; { Run }

procedure TM6502.Pause;
begin
  CPUisRunning := false;
  ForcePause := true;
end; { Pause }

procedure TM6502.Stop;
begin
  CPUisRunning := false;
  //if Assigned(CPUThread) then
  //  CPUThread.Suspend;
end; { Stop }

{ Non-maskable Interrupt }
procedure TM6502.NMI;
begin
 MemoryWrite(S2, Hi(PC));
 system.dec(S2);
 MemoryWrite(S2, Lo(PC));
 system.dec(S2);
 MemoryWrite(S2, P);
 system.dec(S2);
 P  := P or I_FLAG;
 PC := MemoryReadWord(INT_NMI_ADDR);
end; { NMI }

{ maskable Interrupt }
procedure TM6502.IRQ;
begin
 MemoryWrite(S2, Hi(PC));
 system.dec(S2);
 MemoryWrite(S2, Lo(PC));
 system.dec(S2);
 MemoryWrite(S2, P);
 system.dec(S2);
 P  := P or I_FLAG;
 PC := MemoryReadWord(INT_IRQ_ADDR);
end; { IRQ }

function TM6502.MemoryReadWord(Addr : word) : word;
begin
  Result := MemoryReadByte(Addr) + (MemoryReadByte(Addr + 1) shl 8);
end; { MemoryReadWord }

function TM6502.DAsm(var addr : word) : string;
var
  s : string;
  opcode : byte;
  c : word;
begin
 opcode := MemoryReadOpcode(addr);
 case opcode of
  $00: s := 'BRK';
  $01: begin
         system.inc(addr);
         s := 'ORA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $05: begin
         system.inc(addr);
         s := 'ORA $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $09: begin
         system.inc(addr);
         s := 'ORA #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $0D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ORA $' + IntToHex(c, 4);
       end;
  $11: begin
         system.inc(addr);
         s := 'ORA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $12,
  $15: begin
         system.inc(addr);
         s := 'ORA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $19: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ORA $' + IntToHex(c, 4) + ', Y';
       end;
  $1D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ORA $' + IntToHex(c, 4) + ', X';
       end;
  $02,$03,$07,$0B,$0F,$13,$17,$1B,$1F,$22,$23,$27,$2B,$2F,$33,$37,$3B,$3F,
  $42,$43,$44,$47,$4B,$4F,$53,$54,$57,$5B,$5C,$5F,$62,$63,$67,$6B,$6F,$73,
  $77,$7B,$7F,$82,$83,$87,$8B,$8F,$93,$97,$9B,$9F,$A3,$A7,$AB,$AF,$B3,$B7,
  $BB,$BF,$C2,$C3,$C7,$CB,$CF,$D3,$D4,$D7,$DB,$DC,$DF,$E2,$E3,$E7,$EA,$EB,
  $EF,$F3,$F4,$F7,$FB,$FC,$FF: s := 'NOP';
  $04,
  $0C: s := 'TSB';
  $06: begin
         system.inc(addr);
         s := 'ASL ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $0E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ASL ' + IntToHex(c, 4);
       end;
  $16: begin
         system.inc(addr);
         s := 'ASL ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $1E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ASL ' + IntToHex(c, 4) + ', X';
       end;
  $08: s := 'PHP';
  $0A: s := 'ASL A';
  $10: begin
         system.inc(addr);
         s := 'BPL ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $14,
  $1C: s := 'TRB';
  $18: s := 'CLC';
  $1A: s := 'INA';
  $20: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'JSR $' + IntToHex(c, 4);
       end;
  $21: begin
         system.inc(addr);
         s := 'AND (' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $25: begin
         system.inc(addr);
         s := 'AND ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $29: begin
         system.inc(addr);
         s := 'AND #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $2D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'AND ' + IntToHex(c, 4);
       end;
  $31: begin
         system.inc(addr);
         s := 'AND (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $32,
  $35: begin
         system.inc(addr);
         s := 'AND ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $39: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'AND ' + IntToHex(c, 4) + ', Y';
       end;
  $3D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'AND ' + IntToHex(c, 4) + ', X';
       end;
  $24: begin
         system.inc(addr);
         s := 'BIT ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $2C: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'BIT ' + IntToHex(c, 4);
       end;
  $34,
  $3C,
  $89: s := 'BIT';
  $26: begin
         system.inc(addr);
         s := 'ROL $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $2E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ROL $' + IntToHex(c, 4);
       end;
  $36: begin
         system.inc(addr);
         s := 'ROL $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $3E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ROL $' + IntToHex(c, 4) + ', X';
       end;
  $28: s := 'PLP';
  $2A: s := 'ROL A';
  $30: begin
         system.inc(addr);
         s := 'BMI ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $38: s := 'SEC';
  $3A: s := 'DEA';
  $40: s := 'RTI';
  $41: begin
         system.inc(addr);
         s := 'EOR ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $45: begin
         system.inc(addr);
         s := 'EOR $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $49: begin
         system.inc(addr);
         s := 'EOR #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $4D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'EOR $' + IntToHex(c, 4);
       end;
  $51: begin
         system.inc(addr);
         s := 'EOR ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $52,
  $55: begin
         system.inc(addr);
         s := 'EOR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $59: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'EOR $' + IntToHex(c, 4) + ', Y';
       end;
  $5D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'EOR $' + IntToHex(c, 4) + ', X';
       end;
  $46: begin
         system.inc(addr);
         s := 'LSR $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $4E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LSR $' + IntToHex(c, 4);
       end;
  $56: begin
         system.inc(addr);
         s := 'LSR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $5E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LSR $' + IntToHex(c, 4) + ', X';
       end;
  $48: s := 'PHA';
  $4A: s := 'LSR A';
  $4C: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'JMP $' + IntToHex(c, 4);
       end;
  $6C: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'JMP ($' + IntToHex(c, 4) + ')';
       end;
  $7C: s := 'JMP';
  $50: begin
         system.inc(addr);
         s := 'BVC ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $58: s := 'CLI';
  $5A: s := 'PHY';
  $60: s := 'RTS';
  $61,
  $65: begin
         system.inc(addr);
         s := 'ADC ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $69: begin
         system.inc(addr);
         s := 'ADC #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $6D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ADC ' + IntToHex(c, 4);
       end;
  $71,
  $72,
  $75: begin
         system.inc(addr);
         s := 'ADC ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $79,
  $7D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ADC ' + IntToHex(c, 4) + ', X';
       end;
  $64,
  $74,
  $9C,
  $9E: s := 'STZ';
  $66: begin
         system.inc(addr);
         s := 'ROR $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $6E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ROR $' + IntToHex(c, 4);
       end;
  $76: begin
         system.inc(addr);
         s := 'ROR $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $7E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'ROR $' + IntToHex(c, 4) + ', X';
       end;
  $68: s := 'PLA';
  $6A: s := 'ROR A';
  $70: begin
         system.inc(addr);
         s := 'BVS ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $78: s := 'SEI';
  $7A: s := 'PLY';
  $80: s := 'BRA';
  $81: begin
         system.inc(addr);
         s := 'STA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $85: begin
         system.inc(addr);
         s := 'STA $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $8D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'STA $' + IntToHex(c, 4);
       end;
  $91: begin
         system.inc(addr);
         s := 'STA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $92,
  $95: begin
         system.inc(addr);
         s := 'STA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $99: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'STA $' + IntToHex(c, 4) + ', Y';
       end;
  $9D: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'STA $' + IntToHex(c, 4) + ', X';
       end;
  $84: begin
         system.inc(addr);
         s := 'STY $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $8C: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'STY $' + IntToHex(c, 4);
       end;
  $94: begin
         system.inc(addr);
         s := 'STY $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $86: begin
         system.inc(addr);
         s := 'STX $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $8E: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'STX $' + IntToHex(c, 4);
       end;
  $96: begin
         system.inc(addr);
         s := 'STX $' + IntToHex(MemoryReadOpcode(addr), 2) + ', Y';
       end;
  $88: s := 'DEY';
  $8A: s := 'TXA';
  $90: begin
         system.inc(addr);
         s := 'BCC ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $98: s := 'TYA';
  $9A: s := 'TXS';
  $A0: begin
         system.inc(addr);
         s := 'LDY #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $A4: begin
         system.inc(addr);
         s := 'LDY $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $AC: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDY $' + IntToHex(c, 4);
       end;
  $B4: begin
         system.inc(addr);
         s := 'LDY $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $BC: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDY $' + IntToHex(c, 4) + ', X';
       end;
  $A1: begin
         system.inc(addr);
         s := 'LDA ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $A5: begin
         system.inc(addr);
         s := 'LDA $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $A9: begin
         system.inc(addr);
         s := 'LDA #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $AD: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDA $' + IntToHex(c, 4);
       end;
  $B1: begin
         system.inc(addr);
         s := 'LDA ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $B2,
  $B5: begin
         system.inc(addr);
         s := 'LDA $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $B9: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDA $' + IntToHex(c, 4) + ', Y';
       end;
  $BD: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDA $' + IntToHex(c, 4) + ', X';
       end;
  $A2: begin
         system.inc(addr);
         s := 'LDX #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $A6: begin
         system.inc(addr);
         s := 'LDX $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $AE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDX $' + IntToHex(c, 4);
       end;
  $B6: begin
         system.inc(addr);
         s := 'LDX $' + IntToHex(MemoryReadOpcode(addr), 2) + ', Y';
       end;
  $BE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'LDX $' + IntToHex(c, 4) + ', Y';
       end;
  $A8: s := 'TAY';
  $AA: s := 'TAX';
  $B0: s := 'BCS';
  $B8: s := 'CLV';
  $BA: s := 'TSX';
  $C0: begin
         system.inc(addr);
         s := 'CPY #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $C4: begin
         system.inc(addr);
         s := 'CPY ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $CC: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'CPY ' + IntToHex(c, 4);
       end;
  $C1: begin
         system.inc(addr);
         s := 'CMP (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $C5: begin
         system.inc(addr);
         s := 'CMP ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $C9: begin
         system.inc(addr);
         s := 'CMP #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $CD: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'CMP ' + IntToHex(c, 4);
       end;
  $D1: begin
         system.inc(addr);
         s := 'CMP (' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $D2,
  $D5: begin
         system.inc(addr);
         s := 'CMP ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $D9: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'CMP ' + IntToHex(c, 4) + ', Y';
       end;
  $DD: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'CMP ' + IntToHex(c, 4) + ', X';
       end;
  $C6: begin
         system.inc(addr);
         s := 'DEC ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $CE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'DEC ' + IntToHex(c, 4);
       end;
  $D6: begin
         system.inc(addr);
         s := 'DEC ' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $DE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'DEC ' + IntToHex(c, 4) + ', X';
       end;
  $C8: s := 'INY';
  $CA: s := 'DEX';
  $D0: begin
         system.inc(addr);
         s := 'BNE ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $D8: s := 'CLD';
  $DA: s := 'PHX';
  $E0: begin
         system.inc(addr);
         s := 'CPX #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $E4: begin
         system.inc(addr);
         s := 'CPX ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $EC: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'CPX ' + IntToHex(c, 4);
       end;
  $E1: begin
         system.inc(addr);
         s := 'SBC ($' + IntToHex(MemoryReadOpcode(addr), 2) + ', X)';
       end;
  $E5: begin
         system.inc(addr);
         s := 'SBC $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $E9: begin
         system.inc(addr);
         s := 'SBC #' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $ED: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'SBC $' + IntToHex(c, 4);
       end;
  $F1: begin
         system.inc(addr);
         s := 'SBC ($' + IntToHex(MemoryReadOpcode(addr), 2) + '), Y';
       end;
  $F2,
  $F5: begin
         system.inc(addr);
         s := 'SBC $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $F9: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'SBC $' + IntToHex(c, 4) + ', Y';
       end;
  $FD: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'SBC $' + IntToHex(c, 4) + ', X';
       end;
  $E6: begin
         system.inc(addr);
         s := 'INC $' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $EE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'INC $' + IntToHex(c, 4);
       end;
  $F6: begin
         system.inc(addr);
         s := 'INC $' + IntToHex(MemoryReadOpcode(addr), 2) + ', X';
       end;
  $FE: begin
         system.inc(addr, 2);
         c := MemoryReadWord(addr-1);
         s := 'INC $' + IntToHex(c, 4) + ', X';
       end;
  $E8: s := 'INX';
  $F0: begin
         system.inc(addr);
         s := 'BEQ ' + IntToHex(MemoryReadOpcode(addr), 2);
       end;
  $F8: s := 'SED';
  $FA: s := 'PLX';
 end;
 system.inc(addr);
 Result := s;
end; { DAsm }

procedure TM6502.Save;
var
 f : text;
 i : integer;
begin
  AssignFile(f, 'PIPPO.TXT');
  Rewrite(f);
  for i:=0 to $FF do
    if Used[i]<>0 then
      Writeln(f, format('%:2d - %d',[i, Used[i]]));
  CloseFile(f);
end;

procedure Register;
begin
  RegisterComponents('CPU Emulator', [TM6502]);
end;

end. { M6502 }
